"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const BaseFacebookHttpApi_1 = tslib_1.__importDefault(require("./BaseFacebookHttpApi"));
const FacebookApiHttpRequest_1 = tslib_1.__importDefault(require("./FacebookApiHttpRequest"));
/**
 * This class will make specific requests to the facebook API utilizing their sick http logic implemented by BaseFacebookHttpApi.
 */
class FacebookHttpApi extends BaseFacebookHttpApi_1.default {
    /**
     * @see QFacebookHttpApi::auth
     * @param email
     * @param password
     */
    auth(email, password) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.get(new FacebookApiHttpRequest_1.default("https://b-api.facebook.com/method/auth.login", "auth.login", "authenticate", { email, password })).then(res => {
                if (!res.access_token) {
                    console.dir(res);
                    throw new Error("Access token missing!");
                }
                this.token = res.access_token;
                return res;
            });
        });
    }
    /**
     * @see QFacebookHttpApi::usersQuery
     */
    userQuery(userId) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.get(new FacebookApiHttpRequest_1.default("https://graph.facebook.com/graphql", "get", "UsersQuery", {
                query_id: "10153915107411729",
                query_params: JSON.stringify({
                    "0": [userId],
                    "1": true
                })
            }));
        });
    }
    /**
     * @see QFacebookHttpApi::usersQuery
     */
    usersQuery(userId, count) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.get(new FacebookApiHttpRequest_1.default("https://graph.facebook.com/graphql", "get", "FetchContactsFullQuery", {
                query_id: "10154444360806729",
                query_params: JSON.stringify({
                    "0": [userId],
                    "1": count
                })
            }));
        });
    }
    /**
     * @see QFacebookHttpApi::usersQueryAfter
     * @param cursor
     */
    usersQueryAfter(userId, count, cursor) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.get(new FacebookApiHttpRequest_1.default("https://graph.facebook.com/graphql", "get", "FetchContactsFullWithAfterQuery", {
                query_id: "10154444360816729",
                query_params: JSON.stringify({
                    "0": [userId],
                    "1": cursor,
                    "2": count
                })
            }));
        });
    }
    /**
     * @see QFacebookHttpApi::usersQueryDelta
     * @param deltaCursor
     */
    usersQueryDelta(userId, deltaCursor) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.get(new FacebookApiHttpRequest_1.default("https://graph.facebook.com/graphql", "get", "FetchContactsDeltaQuery", {
                query_id: "10154444360801729",
                query_params: JSON.stringify({
                    "0": deltaCursor,
                    "1": [userId],
                    "2": "500"
                })
            }));
        });
    }
    /**
     * @see QFacebookHttpApi::threadListQuery
     */
    threadListQuery(count) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.get(new FacebookApiHttpRequest_1.default("https://graph.facebook.com/graphql", "get", "ThreadListQuery", {
                query_id: "2000270246651497",
                query_params: JSON.stringify({
                    "11": "true",
                    "23": count,
                    "3": "false",
                    "27": "false",
                    "5": "false",
                    "10": "false",
                    "17": 20,
                    "18": 880,
                    "19": 220,
                    "20": 138,
                    "31": "3"
                })
            }));
        });
    }
    /**
     * @param threadId
     * @see facebook-api.c:3317
     */
    threadQuery(threadId) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.get(new FacebookApiHttpRequest_1.default("https://graph.facebook.com/graphql", "get", "ThreadQuery", {
                query_id: "10153919752036729",
                query_params: JSON.stringify({
                    "0": [threadId],
                    "10": false,
                    "11": false,
                    "13": true // show more details for messaging actors
                })
            }));
        });
    }
    querySeqId() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.get(new FacebookApiHttpRequest_1.default("https://graph.facebook.com/graphql", "get", "", {
                query_id: "10155268192741729",
                query_params: JSON.stringify({ "1": "0" })
            }));
        });
    }
    /**
     * @see QFacebookHttpApi::unreadThreadListQuery
     * @param unreadCount
     */
    unreadThreadListQuery(unreadCount) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.get(new FacebookApiHttpRequest_1.default("https://graph.facebook.com/graphql", "get", "ThreadListQuery", {
                query_id: "10153919752026729",
                query_params: JSON.stringify({
                    "1": unreadCount,
                    "2": true,
                    "12": true,
                    "13": false
                })
            }));
        });
    }
    /**
     * @param mid Message ID
     * @param aid Attachment ID
     */
    getAttachment(mid, aid) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.get(new FacebookApiHttpRequest_1.default("https://api.facebook.com/method/messaging.getAttachment", "messaging.getAttachment", "", { mid, aid }));
        });
    }
    /**
     * @param stickerId
     */
    getSticker(stickerId) {
        return this.get(new FacebookApiHttpRequest_1.default("https://graph.facebook.com/graphql", "get", "FetchStickersWithPreviewsQuery", {
            query_id: "10152877994321729",
            query_params: JSON.stringify({
                "0": [stickerId]
            })
        }));
    }
}
exports.default = FacebookHttpApi;
//# sourceMappingURL=FacebookHttpApi.js.map