/// <reference types="node" />
import { EventEmitter } from "events";
import AuthTokens from "../types/AuthTokens";
import DeviceId from "../types/DeviceId";
import MqttConnection from "./MqttConnection";
import MqttMessage from "./MqttMessage";
declare class MqttApiEmitter extends EventEmitter {
}
/**
 * Handles decoding and sending all sorts of messages used by Facebook Messenger.
 * It utilizes all network primitives defined in the MqttConnection class.
 */
export default class MqttApi {
    connection: MqttConnection;
    _connected: boolean;
    lastMsgId: number;
    tokens: AuthTokens;
    emitter: MqttApiEmitter;
    constructor();
    on(event: any, cb: any): void;
    sendSubscribe(msg: MqttMessage): Promise<void>;
    connect(): Promise<void>;
    sendPing: () => Promise<void>;
    /**
     * Sends a CONNECT mqtt message and binds listeners for recieving messages.
     * @param tokens
     * @param deviceId
     */
    sendConnectMessage(tokens: AuthTokens, deviceId: DeviceId): Promise<void>;
    sendPublish(topic: string, data: string): Promise<void>;
    /**
     * Sends a facebook messenger message to someone.
     */
    sendMessage(threadId: string, message: string): Promise<{}>;
    sendPublishConfirmation(flags: number, publish: any): Promise<void>;
}
export {};
