/// <reference types="node" />
import { EventEmitter } from "events";
import { TLSSocket } from "tls";
import MqttMessage from "./MqttMessage";
import { MqttHeader } from "./MqttPacket";
declare class MqttConnectionEmitter extends EventEmitter {
}
/**
 * Represents an encrypted real-time connection with facebook servers.
 * This class encapsulates all logic which handles communication using the propietary MQTT-like protocol.
 */
export default class MqttConnection {
    toSend: Buffer;
    socket: TLSSocket | null;
    lastHeader: MqttHeader | null;
    emitter: MqttConnectionEmitter;
    decodeBuffer: Buffer;
    connectMsg: any;
    /**
     * Connects to Facebook mqtt servers. The promise is resolved when a secure TLS handshake is established. No CONNECT message is sent yet.
     */
    connect(): Promise<void>;
    readBuffer(data: Buffer): void;
    on(string: any, cb: any): void;
    emitPacket(): void;
    writeMessage(message: MqttMessage): Promise<void>;
    /**
     * Parses a MQTT header.
     * @param data The binary data representing the header.
     */
    readHeader(data: Buffer): MqttHeader;
}
export {};
