"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const thrift_1 = require("thrift");
const zlib = tslib_1.__importStar(require("zlib"));
const MqttMessage_1 = tslib_1.__importDefault(require("../MqttMessage"));
const MqttTypes_1 = require("../MqttTypes");
const MessageTypes_1 = require("./MessageTypes");
const USER_AGENT = "Facebook plugin / LIBFB-JS / [FBAN/Orca-Android;FBAV/148.0.0.5.83;FBPN/com.facebook.orca;FBLC/en_US;FBBV/26040814]";
var FacebookCapFlags;
(function (FacebookCapFlags) {
    FacebookCapFlags[FacebookCapFlags["FB_CP_ACKNOWLEDGED_DELIVERY"] = 1] = "FB_CP_ACKNOWLEDGED_DELIVERY";
    FacebookCapFlags[FacebookCapFlags["FB_CP_PROCESSING_LASTACTIVE_PRESENCEINFO"] = 2] = "FB_CP_PROCESSING_LASTACTIVE_PRESENCEINFO";
    FacebookCapFlags[FacebookCapFlags["FB_CP_EXACT_KEEPALIVE"] = 4] = "FB_CP_EXACT_KEEPALIVE";
    FacebookCapFlags[FacebookCapFlags["FB_CP_REQUIRES_JSON_UNICODE_ESCAPES"] = 8] = "FB_CP_REQUIRES_JSON_UNICODE_ESCAPES";
    FacebookCapFlags[FacebookCapFlags["FB_CP_DELTA_SENT_MESSAGE_ENABLED"] = 16] = "FB_CP_DELTA_SENT_MESSAGE_ENABLED";
    FacebookCapFlags[FacebookCapFlags["FB_CP_USE_ENUM_TOPIC"] = 32] = "FB_CP_USE_ENUM_TOPIC";
    FacebookCapFlags[FacebookCapFlags["FB_CP_SUPPRESS_GETDIFF_IN_CONNECT"] = 64] = "FB_CP_SUPPRESS_GETDIFF_IN_CONNECT";
    FacebookCapFlags[FacebookCapFlags["FB_CP_USE_THRIFT_FOR_INBOX"] = 128] = "FB_CP_USE_THRIFT_FOR_INBOX";
    FacebookCapFlags[FacebookCapFlags["FB_CP_USE_SEND_PINGRESP"] = 256] = "FB_CP_USE_SEND_PINGRESP";
    FacebookCapFlags[FacebookCapFlags["FB_CP_REQUIRE_REPLAY_PROTECTION"] = 512] = "FB_CP_REQUIRE_REPLAY_PROTECTION";
    FacebookCapFlags[FacebookCapFlags["FB_CP_DATA_SAVING_MODE"] = 1024] = "FB_CP_DATA_SAVING_MODE";
    FacebookCapFlags[FacebookCapFlags["FB_CP_TYPING_OFF_WHEN_SENDING_MESSAGE"] = 2048] = "FB_CP_TYPING_OFF_WHEN_SENDING_MESSAGE";
})(FacebookCapFlags || (FacebookCapFlags = {}));
/**
 * Assembles a connect messages sent just after a TLS connection is established.
 */
exports.encodeConnectMessage = (tokens, deviceId) => {
    return new Promise((res, rej) => {
        const trans = new thrift_1.TBufferedTransport();
        trans.onFlush = d => {
            const message = new MqttMessage_1.default();
            const flags = MqttTypes_1.MqttConnectFlag.User |
                MqttTypes_1.MqttConnectFlag.Pass |
                MqttTypes_1.MqttConnectFlag.Clr |
                MqttTypes_1.MqttConnectFlag.QoS1;
            message.writeString("MQTToT");
            message.writeU8(3);
            message.writeU8(flags);
            message.writeU16(60); // KEEP ALIVE
            message.writeRaw(zlib.deflateSync(d));
            message.flags = 0;
            message.type = MessageTypes_1.FacebookMessageType.Connect;
            res(message);
        };
        const proto = new thrift_1.TCompactProtocol(trans);
        // Write client id
        proto.lastFieldId_ = 0;
        proto.writeFieldBegin("none", thrift_1.Thrift.Type.STRING, 1);
        proto.writeString(deviceId.clientId);
        proto.lastFieldId_ = 1;
        proto.writeFieldBegin("none", thrift_1.Thrift.Type.STRUCT, 4);
        // Write user id
        proto.lastFieldId_ = 0;
        proto.writeFieldBegin("none", thrift_1.Thrift.Type.I64, 1);
        proto.writeI64(tokens.uid);
        // Write information
        proto.lastFieldId_ = 1;
        proto.writeFieldBegin("none", thrift_1.Thrift.Type.STRING, 2);
        proto.writeString(USER_AGENT);
        // Write some random int (?)
        proto.lastFieldId_ = 2;
        proto.writeFieldBegin("None", thrift_1.Thrift.Type.I64, 3);
        proto.writeI64(FacebookCapFlags.FB_CP_ACKNOWLEDGED_DELIVERY |
            FacebookCapFlags.FB_CP_PROCESSING_LASTACTIVE_PRESENCEINFO |
            FacebookCapFlags.FB_CP_EXACT_KEEPALIVE |
            FacebookCapFlags.FB_CP_DELTA_SENT_MESSAGE_ENABLED);
        // Write some random int (?)
        proto.lastFieldId_ = 3;
        proto.writeFieldBegin("None", thrift_1.Thrift.Type.I64, 4);
        proto.writeI64(26);
        // Write some random int (?)
        proto.lastFieldId_ = 4;
        proto.writeFieldBegin("None", thrift_1.Thrift.Type.I32, 5);
        proto.writeI32(1);
        // Write no_auto_fg boolean
        proto.lastFieldId_ = 5;
        proto.writeFieldBegin("none", thrift_1.Thrift.Type.BOOL, 6);
        proto.writeBool(true);
        // Write visibility state
        proto.lastFieldId_ = 6;
        proto.writeFieldBegin("none", thrift_1.Thrift.Type.BOOL, 7);
        proto.writeBool(true);
        // Write device id
        proto.lastFieldId_ = 7;
        proto.writeFieldBegin("none", thrift_1.Thrift.Type.STRING, 8);
        proto.writeString(deviceId.deviceId);
        // Write fg boolean
        proto.lastFieldId_ = 8;
        proto.writeFieldBegin("none", thrift_1.Thrift.Type.BOOL, 9);
        proto.writeBool(true);
        // nwt int
        proto.lastFieldId_ = 9;
        proto.writeFieldBegin("none", thrift_1.Thrift.Type.I32, 10);
        proto.writeI32(1);
        // nwst int
        proto.lastFieldId_ = 10;
        proto.writeFieldBegin("none", thrift_1.Thrift.Type.I32, 11);
        proto.writeI32(0);
        // write mqtt id
        proto.lastFieldId_ = 11;
        proto.writeFieldBegin("none", thrift_1.Thrift.Type.I64, 12);
        proto.writeI64(deviceId.mqttId);
        // write some random list
        proto.lastFieldId_ = 12;
        proto.writeFieldBegin("none", thrift_1.Thrift.Type.LIST, 14);
        proto.writeListBegin(thrift_1.Thrift.Type.I32, 0); // wtf(?)
        proto.writeFieldStop();
        // Write token
        proto.lastFieldId_ = 14;
        proto.writeFieldBegin("none", thrift_1.Thrift.Type.STRING, 15);
        proto.writeString(tokens.access_token);
        proto.writeFieldStop();
        proto.writeStructEnd();
        proto.flush();
    });
};
//# sourceMappingURL=Connect.js.map